@@ -32,24 +32,17 @@
 #include "nautilus-file-private.h"
 #include "nautilus-file-utilities.h"
 #include <eel/eel-glib-extensions.h>
-#include <eel/eel-gtk-macros.h>
 #include "nautilus-desktop-directory.h"
-#include <gtk/gtksignal.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs.h>
 #include <string.h>
+#include <gio/gio.h>
 
 struct NautilusDesktopIconFileDetails {
 	NautilusDesktopLink *link;
 };
 
-static void nautilus_desktop_icon_file_init       (gpointer   object,
-						   gpointer   klass);
-static void nautilus_desktop_icon_file_class_init (gpointer   klass);
+G_DEFINE_TYPE(NautilusDesktopIconFile, nautilus_desktop_icon_file, NAUTILUS_TYPE_FILE)
 
-EEL_CLASS_BOILERPLATE (NautilusDesktopIconFile,
-		       nautilus_desktop_icon_file,
-		       NAUTILUS_TYPE_FILE)
 
 static void
 desktop_icon_file_monitor_add (NautilusFile *file,
@@ -69,13 +62,11 @@ desktop_icon_file_monitor_remove (NautilusFile *file,
 		(file->details->directory, file, client);
 }
 
-
 static void
 desktop_icon_file_call_when_ready (NautilusFile *file,
 				   NautilusFileAttributes attributes,
 				   NautilusFileCallback callback,
 				   gpointer callback_data)
-
 {
 	nautilus_directory_call_when_ready_internal
 		(file->details->directory, file,
@@ -101,12 +92,6 @@ desktop_icon_file_check_if_ready (NautilusFile *file,
 		 attributes);
 }
 
-static GnomeVFSFileType
-desktop_icon_file_get_file_type (NautilusFile *file)
-{
-	return GNOME_VFS_FILE_TYPE_REGULAR;
-}			      
-
 static gboolean
 desktop_icon_file_get_item_count (NautilusFile *file, 
 				  guint *count,
@@ -126,7 +111,7 @@ desktop_icon_file_get_deep_counts (NautilusFile *file,
 				   guint *directory_count,
 				   guint *file_count,
 				   guint *unreadable_directory_count,
-				   GnomeVFSFileSize *total_size)
+				   goffset *total_size)
 {
 	if (directory_count != NULL) {
 		*directory_count = 0;
@@ -164,22 +149,20 @@ desktop_icon_file_get_where_string (NautilusFile *file)
 }
 
 static void
-nautilus_desktop_icon_file_init (gpointer object, gpointer klass)
+nautilus_desktop_icon_file_init (NautilusDesktopIconFile *desktop_file)
 {
-	NautilusDesktopIconFile *desktop_file;
-
-	desktop_file = NAUTILUS_DESKTOP_ICON_FILE (object);
-
-	desktop_file->details = g_new0 (NautilusDesktopIconFileDetails, 1);
-}	
+	desktop_file->details =	G_TYPE_INSTANCE_GET_PRIVATE (desktop_file,
+							     NAUTILUS_TYPE_DESKTOP_ICON_FILE,
+							     NautilusDesktopIconFileDetails);
+}
 
 static void
 update_info_from_link (NautilusDesktopIconFile *icon_file)
 {
 	NautilusFile *file;
-	GnomeVFSFileInfo *file_info;
 	NautilusDesktopLink *link;
-	GnomeVFSVolume *volume;
+	char *display_name;
+	GMount *mount;
 	
 	file = NAUTILUS_FILE (icon_file);
 	
@@ -188,46 +171,40 @@ update_info_from_link (NautilusDesktopIconFile *icon_file)
 	if (link == NULL) {
 		return;
 	}
-	
-	file_info = file->details->info;
 
-	gnome_vfs_file_info_clear (file_info);
+	eel_ref_str_unref (file->details->mime_type);
+	file->details->mime_type = eel_ref_str_get_unique ("application/x-nautilus-link");
+	file->details->type = G_FILE_TYPE_REGULAR;
+	file->details->size = 0;
+	file->details->has_permissions = FALSE;
+	file->details->can_read = TRUE;
+	file->details->can_write = TRUE;
 
-	file_info->name = nautilus_desktop_link_get_file_name (link);
-	file_info->mime_type = g_strdup ("application/x-nautilus-link");
-	file_info->type = GNOME_VFS_FILE_TYPE_REGULAR;
-	file_info->flags = GNOME_VFS_FILE_FLAGS_NONE;
-	file_info->link_count = 1;
-	file_info->size = 0;
-	file_info->permissions =
-		GNOME_VFS_PERM_OTHER_WRITE |
-		GNOME_VFS_PERM_GROUP_WRITE |
-		GNOME_VFS_PERM_USER_READ |
-		GNOME_VFS_PERM_OTHER_READ |
-		GNOME_VFS_PERM_GROUP_READ |
-		GNOME_VFS_PERM_ACCESS_READABLE |
-		GNOME_VFS_PERM_ACCESS_WRITABLE;
-	
-	file_info->valid_fields = GNOME_VFS_FILE_INFO_FIELDS_TYPE |
-		GNOME_VFS_FILE_INFO_FIELDS_FLAGS |
-		GNOME_VFS_FILE_INFO_FIELDS_MIME_TYPE |
-		GNOME_VFS_FILE_INFO_FIELDS_SIZE |
-		GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS |
-		GNOME_VFS_FILE_INFO_FIELDS_ACCESS |
-		GNOME_VFS_FILE_INFO_FIELDS_LINK_COUNT;
-
-	volume = nautilus_desktop_link_get_volume (link);
-	nautilus_file_set_volume (file, volume);
-	gnome_vfs_volume_unref (volume);
+	file->details->can_mount = FALSE;
+	file->details->can_unmount = FALSE;
+	file->details->can_eject = FALSE;
+	mount = nautilus_desktop_link_get_mount (link);
+	if (mount) {
+		file->details->can_unmount = g_mount_can_unmount (mount);
+		file->details->can_eject = g_mount_can_eject (mount);
+		g_object_unref (mount);
+	}
 	
 	file->details->file_info_is_up_to_date = TRUE;
 
-	g_free (file->details->display_name);
-	file->details->display_name = nautilus_desktop_link_get_display_name (link);
-	g_free (file->details->custom_icon);
-	file->details->custom_icon = nautilus_desktop_link_get_icon (link);
-	g_free (file->details->activation_uri);
-	file->details->activation_uri = nautilus_desktop_link_get_activation_uri (link);
+	display_name = nautilus_desktop_link_get_display_name (link);
+	nautilus_file_set_display_name (file,
+					display_name, NULL, TRUE);
+	g_free (display_name);
+
+	if (file->details->icon != NULL) {
+		g_object_unref (file->details->icon);
+	}
+	file->details->icon = nautilus_desktop_link_get_icon (link);
+	if (file->details->activation_location) {
+		g_object_unref (file->details->activation_location);
+	}
+	file->details->activation_location = nautilus_desktop_link_get_activation_location (link);
 	file->details->got_link_info = TRUE;
 	file->details->link_info_is_up_to_date = TRUE;
 
@@ -243,7 +220,6 @@ nautilus_desktop_icon_file_update (NautilusDesktopIconFile *icon_file)
 	
 	update_info_from_link (icon_file);
 	file = NAUTILUS_FILE (icon_file);
-	nautilus_file_clear_cached_display_name (file);
 	nautilus_file_changed (file);
 }
 
@@ -275,17 +251,16 @@ nautilus_desktop_icon_file_remove (NautilusDesktopIconFile *icon_file)
 	nautilus_file_unref (file);
 }
 
-
 NautilusDesktopIconFile *
 nautilus_desktop_icon_file_new (NautilusDesktopLink *link)
 {
 	NautilusFile *file;
 	NautilusDirectory *directory;
 	NautilusDesktopIconFile *icon_file;
-	char *name;
 	GList list;
-	
-	directory = nautilus_directory_get (EEL_DESKTOP_URI);
+	char *name;
+
+	directory = nautilus_directory_get_by_uri (EEL_DESKTOP_URI);
 
 	file = NAUTILUS_FILE (g_object_new (NAUTILUS_TYPE_DESKTOP_ICON_FILE, NULL));
 
@@ -299,13 +274,12 @@ nautilus_desktop_icon_file_new (NautilusDesktopLink *link)
 	icon_file = NAUTILUS_DESKTOP_ICON_FILE (file);
 	icon_file->details->link = link;
 
-	file->details->info = gnome_vfs_file_info_new ();
 	name = nautilus_desktop_link_get_file_name (link);
-	file->details->relative_uri = gnome_vfs_escape_string (name);
+	file->details->name = eel_ref_str_new (name);
 	g_free (name);
 
 	update_info_from_link (icon_file);
-	
+
 	nautilus_directory_add_file (directory, file);
 
 	list.data = file;
@@ -316,7 +290,6 @@ nautilus_desktop_icon_file_new (NautilusDesktopLink *link)
 	return icon_file;
 }
 
-
 /* Note: This can return NULL if the link was recently removed (i.e. unmounted) */
 NautilusDesktopLink *
 nautilus_desktop_icon_file_get_link (NautilusDesktopIconFile *icon_file)
@@ -328,36 +301,25 @@ nautilus_desktop_icon_file_get_link (NautilusDesktopIconFile *icon_file)
 }
 
 static void
-desktop_icon_file_finalize (GObject *object)
-{
-	NautilusDesktopIconFile *desktop_file;
-
-	desktop_file = NAUTILUS_DESKTOP_ICON_FILE (object);
-
-	g_free (desktop_file->details);
-
-	EEL_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
-}
-
-static void
-nautilus_desktop_icon_file_class_init (gpointer klass)
+nautilus_desktop_icon_file_class_init (NautilusDesktopIconFileClass *klass)
 {
 	GObjectClass *object_class;
 	NautilusFileClass *file_class;
 
 	object_class = G_OBJECT_CLASS (klass);
 	file_class = NAUTILUS_FILE_CLASS (klass);
-	
-	object_class->finalize = desktop_icon_file_finalize;
 
+	file_class->default_file_type = G_FILE_TYPE_DIRECTORY;
+	
 	file_class->monitor_add = desktop_icon_file_monitor_add;
 	file_class->monitor_remove = desktop_icon_file_monitor_remove;
 	file_class->call_when_ready = desktop_icon_file_call_when_ready;
 	file_class->cancel_call_when_ready = desktop_icon_file_cancel_call_when_ready;
 	file_class->check_if_ready = desktop_icon_file_check_if_ready;
-	file_class->get_file_type = desktop_icon_file_get_file_type;
 	file_class->get_item_count = desktop_icon_file_get_item_count;
 	file_class->get_deep_counts = desktop_icon_file_get_deep_counts;
 	file_class->get_date = desktop_icon_file_get_date;
 	file_class->get_where_string = desktop_icon_file_get_where_string;
+
+	g_type_class_add_private (object_class, sizeof(NautilusDesktopIconFileDetails));
 }
