@@ -30,22 +30,16 @@
  */
 
 typedef enum {
-	NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI = 1 << 0,
-	NAUTILUS_FILE_ATTRIBUTE_CAPABILITIES = 1 << 1,
-	NAUTILUS_FILE_ATTRIBUTE_CUSTOM_ICON = 1 << 2,
-	NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS = 1 << 3,
-	NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_COUNT = 1 << 4,
-	NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES = 1 << 5,
-	NAUTILUS_FILE_ATTRIBUTE_FILE_TYPE = 1 << 6,
-	NAUTILUS_FILE_ATTRIBUTE_IS_DIRECTORY = 1 << 7,
-	NAUTILUS_FILE_ATTRIBUTE_METADATA = 1 << 8,
-	NAUTILUS_FILE_ATTRIBUTE_MIME_TYPE = 1 << 9,
-	NAUTILUS_FILE_ATTRIBUTE_TOP_LEFT_TEXT = 1 << 10,
-	NAUTILUS_FILE_ATTRIBUTE_LARGE_TOP_LEFT_TEXT = 1 << 11,
-	NAUTILUS_FILE_ATTRIBUTE_DISPLAY_NAME = 1 << 12,
-	NAUTILUS_FILE_ATTRIBUTE_VOLUMES = 1 << 13,
-	NAUTILUS_FILE_ATTRIBUTE_EXTENSION_INFO = 1 << 14,
-	NAUTILUS_FILE_ATTRIBUTE_SLOW_MIME_TYPE = 1 << 15
+	NAUTILUS_FILE_ATTRIBUTE_INFO = 1 << 0, /* All standard info */
+	NAUTILUS_FILE_ATTRIBUTE_LINK_INFO = 1 << 1, /* info from desktop links */
+	NAUTILUS_FILE_ATTRIBUTE_DEEP_COUNTS = 1 << 2,
+	NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_COUNT = 1 << 3,
+	NAUTILUS_FILE_ATTRIBUTE_DIRECTORY_ITEM_MIME_TYPES = 1 << 4,
+	NAUTILUS_FILE_ATTRIBUTE_METADATA = 1 << 5,
+	NAUTILUS_FILE_ATTRIBUTE_TOP_LEFT_TEXT = 1 << 6,
+	NAUTILUS_FILE_ATTRIBUTE_LARGE_TOP_LEFT_TEXT = 1 << 7,
+	NAUTILUS_FILE_ATTRIBUTE_EXTENSION_INFO = 1 << 8,
+	NAUTILUS_FILE_ATTRIBUTE_THUMBNAIL = 1 << 9,
 } NautilusFileAttributes;
 
 #endif /* NAUTILUS_FILE_ATTRIBUTES_H */
