@@ -27,6 +27,7 @@
 #include "nautilus-merged-directory.h"
 
 #include "nautilus-directory-private.h"
+#include "nautilus-directory-notify.h"
 #include "nautilus-file.h"
 #include <eel/eel-glib-extensions.h>
 #include <gtk/gtksignal.h>
@@ -44,7 +45,7 @@ typedef struct {
 	NautilusDirectoryCallback callback;
 	gpointer callback_data;
 
-	GList *wait_for_attributes;
+	NautilusFileAttributes wait_for_attributes;
 	gboolean wait_for_file_list;
 
 	GList *non_ready_directories;
@@ -56,7 +57,7 @@ typedef struct {
 
 	gboolean monitor_hidden_files;
 	gboolean monitor_backup_files;
-	GList *monitor_attributes;
+	NautilusFileAttributes monitor_attributes;
 } MergedMonitor;
 
 enum {
@@ -99,7 +100,6 @@ merged_callback_destroy (MergedCallback *merged_callback)
 	g_assert (merged_callback != NULL);
 	g_assert (NAUTILUS_IS_MERGED_DIRECTORY (merged_callback->merged));
 
-	eel_g_list_free_deep (merged_callback->wait_for_attributes);
 	g_list_free (merged_callback->non_ready_directories);
 	nautilus_file_list_free (merged_callback->merged_file_list);
 	g_free (merged_callback);
@@ -158,7 +158,7 @@ directory_ready_callback (NautilusDirectory *directory,
 
 static void
 merged_call_when_ready (NautilusDirectory *directory,
-			GList *file_attributes,
+			NautilusFileAttributes file_attributes,
 			gboolean wait_for_file_list,
 			NautilusDirectoryCallback callback,
 			gpointer callback_data)
@@ -182,7 +182,7 @@ merged_call_when_ready (NautilusDirectory *directory,
 	merged_callback->merged = merged;
 	merged_callback->callback = callback;
 	merged_callback->callback_data = callback_data;
-	merged_callback->wait_for_attributes = eel_g_str_list_copy (file_attributes);
+	merged_callback->wait_for_attributes = file_attributes;
 	merged_callback->wait_for_file_list = wait_for_file_list;
 	for (node = merged->details->directories; node != NULL; node = node->next) {
 		merged_callback->non_ready_directories = g_list_prepend
@@ -254,12 +254,12 @@ build_merged_callback_list (NautilusDirectory *directory,
 /* Create a monitor on each of the directories in the list. */
 static void
 merged_monitor_add (NautilusDirectory *directory,
-			 gconstpointer client,
-			 gboolean monitor_hidden_files,
-			 gboolean monitor_backup_files,
-			 GList *file_attributes,
-			 NautilusDirectoryCallback callback,
-			 gpointer callback_data)
+		    gconstpointer client,
+		    gboolean monitor_hidden_files,
+		    gboolean monitor_backup_files,
+		    NautilusFileAttributes file_attributes,
+		    NautilusDirectoryCallback callback,
+		    gpointer callback_data)
 {
 	NautilusMergedDirectory *merged;
 	MergedMonitor *monitor;
@@ -274,7 +274,6 @@ merged_monitor_add (NautilusDirectory *directory,
 	monitor = g_hash_table_lookup (merged->details->monitors, client);
 	if (monitor != NULL) {
 		g_assert (monitor->merged == merged);
-		eel_g_list_free_deep (monitor->monitor_attributes);
 	} else {
 		monitor = g_new0 (MergedMonitor, 1);
 		monitor->merged = merged;
@@ -283,7 +282,7 @@ merged_monitor_add (NautilusDirectory *directory,
 	}
 	monitor->monitor_hidden_files = monitor_hidden_files;
 	monitor->monitor_backup_files = monitor_backup_files;
-	monitor->monitor_attributes = eel_g_str_list_copy (file_attributes);
+	monitor->monitor_attributes = file_attributes;
 	
 	/* Call through to the real directory add calls. */
 	merged_callback_list = NULL;
@@ -310,7 +309,6 @@ merged_monitor_destroy (NautilusMergedDirectory *merged, MergedMonitor *monitor)
 		nautilus_directory_file_monitor_remove (node->data, monitor);
 	}
 
-	eel_g_list_free_deep (monitor->monitor_attributes);
 	g_free (monitor);
 }
 
@@ -400,6 +398,30 @@ merged_is_not_empty (NautilusDirectory *directory)
 	return FALSE;
 }
 
+static GList *
+merged_get_file_list (NautilusDirectory *directory)
+{
+	GList *dirs_file_list, *merged_dir_file_list;
+	GList *dir_list;
+	GList *cur_node;
+
+	dirs_file_list = NULL;
+	dir_list = NAUTILUS_MERGED_DIRECTORY (directory)->details->directories;
+
+	for (cur_node = dir_list; cur_node != NULL; cur_node = cur_node->next) {
+		NautilusDirectory *cur_dir;
+
+		cur_dir = NAUTILUS_DIRECTORY (cur_node->data);
+		dirs_file_list = g_list_concat (dirs_file_list,
+						 nautilus_directory_get_file_list (cur_dir));
+	}
+
+	merged_dir_file_list = GNOME_CALL_PARENT_WITH_DEFAULT
+				(NAUTILUS_DIRECTORY_CLASS, get_file_list, (directory), NULL);
+
+	return g_list_concat (dirs_file_list, merged_dir_file_list);
+}
+
 static void
 forward_files_added_cover (NautilusDirectory *real_directory,
 			   GList *files,
@@ -518,6 +540,31 @@ monitor_remove_directory (gpointer key,
 		(NAUTILUS_DIRECTORY (callback_data), value);
 }
 
+static void
+real_directory_notify_files_removed (NautilusDirectory *real_directory)
+{
+	GList *files, *l;
+
+	files = nautilus_directory_get_file_list (real_directory);
+
+	for (l = files; l; l = l->next) {
+		NautilusFile *file;
+		char *uri;
+
+		file = NAUTILUS_FILE (l->data);
+		uri = nautilus_file_get_uri (file);
+		nautilus_file_unref (file);
+
+		l->data = uri;
+	}
+
+	if (files) {
+		nautilus_directory_notify_files_removed_by_uri (files);
+	}
+
+	eel_g_list_free_deep (files);
+}
+
 static void
 merged_remove_real_directory (NautilusMergedDirectory *merged,
 			      NautilusDirectory *real_directory)
@@ -526,6 +573,9 @@ merged_remove_real_directory (NautilusMergedDirectory *merged,
 	g_return_if_fail (NAUTILUS_IS_DIRECTORY (real_directory));
 	g_return_if_fail (g_list_find (merged->details->directories, real_directory) != NULL);
 
+	/* Since the real directory will be going away, act as if files were removed */
+	real_directory_notify_files_removed (real_directory);
+
 	/* Remove this directory from callbacks and monitors. */
 	eel_g_hash_table_safe_for_each (merged->details->callbacks,
 					merged_callback_remove_directory_cover,
@@ -624,6 +674,10 @@ nautilus_merged_directory_class_init (NautilusMergedDirectoryClass *class)
 	directory_class->force_reload = merged_force_reload;
  	directory_class->are_all_files_seen = merged_are_all_files_seen;
 	directory_class->is_not_empty = merged_is_not_empty;
+	/* Override get_file_list so that we can return a list that includes
+	 * the files from each of the directories in NautilusMergedDirectory->details->directories.
+         */
+	directory_class->get_file_list = merged_get_file_list;
 
 	class->add_real_directory = merged_add_real_directory;
 	class->remove_real_directory = merged_remove_real_directory;
