@@ -44,7 +44,6 @@
 #include <eel/eel-gconf-extensions.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-preferences-glade.h>
-#include <eel/eel-string-list.h>
 
 #include <libnautilus-private/nautilus-column-chooser.h>
 #include <libnautilus-private/nautilus-column-utilities.h>
@@ -162,17 +161,6 @@ static const char * const icon_captions_components[] = {
 	NULL
 };
 
-static GladeXML *
-nautilus_file_management_properties_dialog_create (void)
-{
-	GladeXML *xml_dialog;
-
-	xml_dialog = glade_xml_new (GLADEDIR "/nautilus-file-management-properties.glade",
-				    NULL, NULL);
-
-	return xml_dialog;
-}
-
 static void
 nautilus_file_management_properties_size_group_create (GladeXML *xml_dialog,
 						       char *prefix,
@@ -267,18 +255,25 @@ static void
 columns_changed_callback (NautilusColumnChooser *chooser,
 			  gpointer callback_data)
 {
-	GList *visible_columns;
-	GList *column_order;
-	
+	char **visible_columns;
+	char **column_order;
+
 	nautilus_column_chooser_get_settings (NAUTILUS_COLUMN_CHOOSER (chooser),
 					      &visible_columns,
 					      &column_order);
 
-	eel_preferences_set_string_glist (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS, visible_columns);
-	eel_preferences_set_string_glist (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER, column_order);
+	eel_preferences_set_string_array (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS, visible_columns);
+	eel_preferences_set_string_array (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER, column_order);
 
-	eel_g_list_free_deep (visible_columns);
-	eel_g_list_free_deep (column_order);
+	g_strfreev (visible_columns);
+	g_strfreev (column_order);
+}
+
+static void
+free_column_names_array (GPtrArray *column_names)
+{
+	g_ptr_array_foreach (column_names, (GFunc) g_free, NULL);
+	g_ptr_array_free (column_names, TRUE);
 }
 
 static void
@@ -286,12 +281,12 @@ create_icon_caption_combo_box_items (GtkComboBox *combo_box,
 			             GList *columns)
 {
 	GList *l;
-	EelStringList *column_names;
+	GPtrArray *column_names;
+
+	column_names = g_ptr_array_new ();
 
-	column_names = eel_string_list_new (FALSE);
-	
 	gtk_combo_box_append_text (combo_box, _("None"));
-	eel_string_list_insert (column_names, "none");
+	g_ptr_array_add (column_names, g_strdup ("none"));
 
 	for (l = columns; l != NULL; l = l->next) {
 		NautilusColumn *column;
@@ -299,7 +294,7 @@ create_icon_caption_combo_box_items (GtkComboBox *combo_box,
 		char *label;
 
 		column = NAUTILUS_COLUMN (l->data);
-		
+
 		g_object_get (G_OBJECT (column), 
 			      "name", &name, "label", &label, 
 			      NULL);
@@ -310,48 +305,50 @@ create_icon_caption_combo_box_items (GtkComboBox *combo_box,
 			g_free (label);
 			continue;
 		}
-		
-		gtk_combo_box_append_text (combo_box, label);
-		eel_string_list_insert (column_names, name);
 
-		g_free (name);
+		gtk_combo_box_append_text (combo_box, label);
+		g_ptr_array_add (column_names, name);
+
 		g_free (label);
 	}
 	g_object_set_data_full (G_OBJECT (combo_box), "column_names",
 			        column_names,
-			        (GDestroyNotify) eel_string_list_free);
+			        (GDestroyNotify) free_column_names_array);
 }
 
 static void
 icon_captions_changed_callback (GtkComboBox *combo_box,
 				gpointer user_data)
 {
-	GList *captions;
+	GPtrArray *captions;
 	GladeXML *xml;
 	int i;
 	
 	xml = GLADE_XML (user_data);
 
-	captions = NULL;
+	captions = g_ptr_array_new ();
 
 	for (i = 0; icon_captions_components[i] != NULL; i++) {
 		GtkWidget *combo_box;
 		int active;
-		EelStringList *column_names;
+		GPtrArray *column_names;
 		char *name;
-		
+
 		combo_box = glade_xml_get_widget
 			    (GLADE_XML (xml), icon_captions_components[i]);
 		active = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
 
-		column_names = g_object_get_data (G_OBJECT (combo_box), "column_names");
-		
-		name = eel_string_list_nth (column_names, active);
-		captions = g_list_prepend (captions, name);
+		column_names = g_object_get_data (G_OBJECT (combo_box),
+						  "column_names");
+
+		name = g_ptr_array_index (column_names, active);
+		g_ptr_array_add (captions, name);
 	}
-	captions = g_list_reverse (captions);
-	eel_preferences_set_string_glist (NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS, captions);
-	eel_g_list_free_deep (captions);
+	g_ptr_array_add (captions, NULL);
+
+	eel_preferences_set_string_array (NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS,
+					  (char **)captions->pdata);
+	g_ptr_array_free (captions, TRUE);
 }
 
 static void
@@ -361,8 +358,8 @@ update_caption_combo_box (GladeXML *xml,
 {
 	GtkWidget *combo_box;
 	int i;
-	EelStringList *column_names;
-	
+	GPtrArray *column_names;
+
 	combo_box = glade_xml_get_widget (xml, combo_box_name);
 
 	g_signal_handlers_block_by_func
@@ -373,8 +370,8 @@ update_caption_combo_box (GladeXML *xml,
 	column_names = g_object_get_data (G_OBJECT (combo_box), 
 					  "column_names");
 
-	for (i = 0; i < eel_string_list_get_length (column_names); i++) {
-		if (!strcmp (name, eel_string_list_peek_nth (column_names, i))) {
+	for (i = 0; i < column_names->len; ++i) {
+		if (!strcmp (name, g_ptr_array_index (column_names, i))) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), i);
 			break;
 		}
@@ -389,28 +386,29 @@ update_caption_combo_box (GladeXML *xml,
 static void
 update_icon_captions_from_gconf (GladeXML *xml)
 {
-	GList *captions;
-	int i;
-	GList *l;
-	char *data;
+	char **captions;
+	int i, j;
 
-	captions = eel_preferences_get_string_glist (NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS);
+	captions = eel_preferences_get_string_array (NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS);
 
-	for (l = captions, i = 0; 
+	for (i = 0, j = 0; 
 	     icon_captions_components[i] != NULL;
 	     i++) {
-		if (l != NULL) {
-			data = l->data;
-			l = l->next;
+		char *data;
+
+		if (captions[j]) {
+			data = captions[j];
+			++j;
 		} else {
 			data = "none";
 		}
-		
+
 		update_caption_combo_box (xml, 
 					  icon_captions_components[i],
 					  data);
 	}
-	eel_g_list_free_deep (captions);
+
+	g_strfreev (captions);
 }
 
 static void
@@ -472,19 +470,18 @@ create_date_format_menu (GladeXML *xml_dialog)
 static void
 set_columns_from_gconf (NautilusColumnChooser *chooser)
 {
-	GList *visible_columns;
-	GList *column_order;
+	char **visible_columns;
+	char **column_order;
 	
-	visible_columns = eel_preferences_get_string_glist (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS);
-	column_order = eel_preferences_get_string_glist (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER);
+	visible_columns = eel_preferences_get_string_array (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS);
+	column_order = eel_preferences_get_string_array (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER);
 
 	nautilus_column_chooser_set_settings (NAUTILUS_COLUMN_CHOOSER (chooser), 
 					      visible_columns,
 					      column_order);
 
-
-	eel_g_list_free_deep (visible_columns);
-	eel_g_list_free_deep (column_order);
+	g_strfreev (visible_columns);
+	g_strfreev (column_order);
 }
 
 static void 
@@ -665,7 +662,8 @@ nautilus_file_management_properties_dialog_show (GCallback close_callback, GtkWi
 {
 	GladeXML *xml_dialog;
 
-	xml_dialog = nautilus_file_management_properties_dialog_create ();
+	xml_dialog = glade_xml_new (GLADEDIR "/nautilus-file-management-properties.glade",
+				    NULL, NULL);
 	
 	g_signal_connect (G_OBJECT (glade_xml_get_widget (xml_dialog, "file_management_dialog")),
 			  "response", close_callback, NULL);
@@ -673,4 +671,6 @@ nautilus_file_management_properties_dialog_show (GCallback close_callback, GtkWi
 			  "delete_event", G_CALLBACK (delete_event_callback), close_callback);
 
 	nautilus_file_management_properties_dialog_setup (xml_dialog, window);
+
+	g_object_unref (xml_dialog);
 }
