@@ -37,6 +37,7 @@
 #include <bonobo-activation/bonobo-activation.h>
 #include <bonobo/bonobo-main.h>
 #include <dlfcn.h>
+#include <signal.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-self-checks.h>
@@ -49,7 +50,7 @@
 #include <libgnome/gnome-init.h>
 #include <libgnomeui/gnome-ui-init.h>
 #include <libgnomeui/gnome-client.h>
-#include <libgnomevfs/gnome-vfs-init.h>
+#include <libnautilus-private/nautilus-debug-log.h>
 #include <libnautilus-private/nautilus-directory-metafile.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-lib-self-check-functions.h>
@@ -61,6 +62,10 @@
 #include <string.h>
 #include <unistd.h>
 
+#ifdef HAVE_EXEMPI
+#include <exempi/xmp.h>
+#endif
+
 /* Keeps track of everyone who wants the main event loop kept active */
 static GSList *event_loop_registrants;
 
@@ -79,7 +84,7 @@ quit_if_in_main_loop (gpointer callback_data)
 
 	level = gtk_main_level ();
 
-	/* We can be called even outside the main loop by gnome_vfs_shutdown,
+	/* We can be called even outside the main loop,
 	 * so check that we are in a loop before calling quit.
 	 */
 	if (level != 0) {
@@ -198,6 +203,166 @@ slowly_and_stupidly_obtain_timestamp (Display *xdisplay)
 	return event.xproperty.time;
 }
 
+static void
+dump_debug_log (void)
+{
+	char *filename;
+
+	filename = g_build_filename (g_get_home_dir (), "nautilus-debug-log.txt", NULL);
+	nautilus_debug_log_dump (filename, NULL); /* NULL GError */
+	g_free (filename);
+}
+
+static int debug_log_pipes[2];
+
+static gboolean
+debug_log_io_cb (GIOChannel *io, GIOCondition condition, gpointer data)
+{
+	char a;
+
+	while (read (debug_log_pipes[0], &a, 1) != 1)
+		;
+
+	nautilus_debug_log (TRUE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "user requested dump of debug log");
+
+	dump_debug_log ();
+	return FALSE;
+}
+
+static void
+sigusr1_handler (int sig)
+{
+	while (write (debug_log_pipes[1], "a", 1) != 1)
+		;
+}
+
+/* This is totally broken as we're using non-signal safe
+ * calls in sigfatal_handler. Disable by default. */
+#ifdef USE_SEGV_HANDLER
+
+/* sigaction structures for the old handlers of these signals */
+static struct sigaction old_segv_sa;
+static struct sigaction old_abrt_sa;
+static struct sigaction old_trap_sa;
+static struct sigaction old_fpe_sa;
+static struct sigaction old_bus_sa;
+
+static void
+sigfatal_handler (int sig)
+{
+	void (* func) (int);
+
+	/* FIXME: is this totally busted?  We do malloc() inside these functions,
+	 * and yet we are inside a signal handler...
+	 */
+	nautilus_debug_log (TRUE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "debug log dumped due to signal %d", sig);
+	dump_debug_log ();
+
+	switch (sig) {
+	case SIGSEGV:
+		func = old_segv_sa.sa_handler;
+		break;
+
+	case SIGABRT:
+		func = old_abrt_sa.sa_handler;
+		break;
+
+	case SIGTRAP:
+		func = old_trap_sa.sa_handler;
+		break;
+
+	case SIGFPE:
+		func = old_fpe_sa.sa_handler;
+		break;
+
+	case SIGBUS:
+		func = old_bus_sa.sa_handler;
+		break;
+
+	default:
+		func = NULL;
+		break;
+	}
+
+	/* this scares me */
+	if (func != NULL && func != SIG_IGN && func != SIG_DFL)
+		(* func) (sig);
+}
+#endif
+
+static void
+setup_debug_log_signals (void)
+{
+	struct sigaction sa;
+	GIOChannel *io;
+
+	if (pipe (debug_log_pipes) == -1)
+		g_error ("Could not create pipe() for debug log");
+
+	io = g_io_channel_unix_new (debug_log_pipes[0]);
+	g_io_add_watch (io, G_IO_IN, debug_log_io_cb, NULL);
+
+	sa.sa_handler = sigusr1_handler;
+	sigemptyset (&sa.sa_mask);
+	sa.sa_flags = 0;
+	sigaction (SIGUSR1, &sa, NULL);
+
+	/* This is totally broken as we're using non-signal safe
+	 * calls in sigfatal_handler. Disable by default. */
+#ifdef USE_SEGV_HANDLER
+	sa.sa_handler = sigfatal_handler;
+	sigemptyset (&sa.sa_mask);
+	sa.sa_flags = 0;
+
+	sigaction(SIGSEGV, &sa, &old_segv_sa);
+	sigaction(SIGABRT, &sa, &old_abrt_sa);
+	sigaction(SIGTRAP, &sa, &old_trap_sa);
+	sigaction(SIGFPE,  &sa, &old_fpe_sa);
+	sigaction(SIGBUS,  &sa, &old_bus_sa);
+#endif
+}
+
+static GLogFunc default_log_handler;
+
+static void
+log_override_cb (const gchar   *log_domain,
+		 GLogLevelFlags log_level,
+		 const gchar   *message,
+		 gpointer       user_data)
+{
+	gboolean is_debug;
+	gboolean is_milestone;
+
+	is_debug = ((log_level & G_LOG_LEVEL_DEBUG) != 0);
+	is_milestone = !is_debug;
+
+	nautilus_debug_log (is_milestone, NAUTILUS_DEBUG_LOG_DOMAIN_GLOG, "%s", message);
+
+	if (!is_debug)
+		(* default_log_handler) (log_domain, log_level, message, user_data);
+}
+
+static void
+setup_debug_log_glog (void)
+{
+	default_log_handler = g_log_set_default_handler (log_override_cb, NULL);
+}
+
+static void
+setup_debug_log (void)
+{
+	char *config_filename;
+
+	config_filename = g_build_filename (g_get_home_dir (), "nautilus-debug-log.conf", NULL);
+	nautilus_debug_log_load_configuration (config_filename, NULL); /* NULL GError */
+	g_free (config_filename);
+
+	setup_debug_log_signals ();
+	setup_debug_log_glog ();
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -242,6 +407,8 @@ main (int argc, char *argv[])
 		{ NULL }
 	};
 
+	g_thread_init (NULL);
+
 	setlocale (LC_ALL, "");
 
 	if (g_getenv ("NAUTILUS_DEBUG") != NULL) {
@@ -280,6 +447,10 @@ main (int argc, char *argv[])
 
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
 
+#ifdef HAVE_EXEMPI
+	xmp_init();
+#endif
+
 	program = gnome_program_init ("nautilus", VERSION,
 				      LIBGNOMEUI_MODULE, argc, argv,
 				      GNOME_PROGRAM_STANDARD_PROPERTIES,
@@ -287,6 +458,12 @@ main (int argc, char *argv[])
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Nautilus"),
 				      NULL);
 
+	/* We do this after gnome_program_init(), since that function sets up
+	 * its own handler for SIGSEGV and others --- we want to chain to those
+	 * handlers.
+	 */
+	setup_debug_log ();
+
 	if (session_to_load != NULL) {
 		no_default_window = TRUE;
 	}
@@ -358,8 +535,6 @@ main (int argc, char *argv[])
 #ifndef NAUTILUS_OMIT_SELF_CHECK
 		/* Run the checks (each twice) for nautilus and libnautilus-private. */
 
-		nautilus_directory_use_self_contained_metafile_factory ();
-
 		nautilus_run_self_checks ();
 		nautilus_run_lib_self_checks ();
 		eel_exit_if_self_checks_failed ();
@@ -389,17 +564,8 @@ main (int argc, char *argv[])
 		gtk_main ();
 	}
 
-	/* This has to be done before gnome_vfs_shutdown, because
- 	 * it might call nautilus_file_get_uri() which might call
- 	 * gnome_vfs_uri_append_string()
- 	 */
  	eel_debug_shut_down ();
 
-	gnome_vfs_shutdown ();
-
-	/* This has to be done after gnome_vfs_shutdown, because shutdown
-	 * can call pending completion callbacks which reference application.
-	 */
 	if (application != NULL) {
 		bonobo_object_unref (application);
 	}
