@@ -51,11 +51,11 @@
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-module.h>
+#include <libnautilus-private/nautilus-file.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file-operations.h>
 #include <libnautilus-private/nautilus-trash-monitor.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-volume-monitor.h>
+#include <gio/gio.h>
 
 #include "nautilus-bookmark-list.h"
 #include "nautilus-places-sidebar.h"
@@ -73,6 +73,7 @@ typedef struct {
 	GtkTreeModel       *filter_model;
 	NautilusWindowInfo *window;
 	NautilusBookmarkList *bookmarks;
+	GVolumeMonitor *volume_monitor;
 
 	/* DnD */
 	GList     *drag_list;
@@ -86,6 +87,7 @@ typedef struct {
 	GtkWidget *popup_menu_mount_item;
 	GtkWidget *popup_menu_unmount_item;
 	GtkWidget *popup_menu_eject_item;
+	GtkWidget *popup_menu_rescan_item;
 	GtkWidget *popup_menu_format_item;
 	GtkWidget *popup_menu_empty_trash_item;
 } NautilusPlacesSidebar;
@@ -107,6 +109,7 @@ enum {
 	PLACES_SIDEBAR_COLUMN_URI,
 	PLACES_SIDEBAR_COLUMN_DRIVE,
 	PLACES_SIDEBAR_COLUMN_VOLUME,
+	PLACES_SIDEBAR_COLUMN_MOUNT,
 	PLACES_SIDEBAR_COLUMN_NAME,
 	PLACES_SIDEBAR_COLUMN_ICON,
 	PLACES_SIDEBAR_COLUMN_INDEX,
@@ -183,21 +186,27 @@ G_DEFINE_TYPE_WITH_CODE (NautilusPlacesSidebarProvider, nautilus_places_sidebar_
 			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_SIDEBAR_PROVIDER,
 						sidebar_provider_iface_init));
 
-
 static GtkTreeIter
 add_place (NautilusPlacesSidebar *sidebar,
 	   PlaceType place_type,
 	   const char *name,
-	   const char *icon,
+	   GIcon *icon,
 	   const char *uri,
-	   GnomeVFSDrive *drive,
-	   GnomeVFSVolume *volume,
+	   GDrive *drive,
+	   GVolume *volume,
+	   GMount *mount,
 	   const int index)
 {
 	GdkPixbuf            *pixbuf;
 	GtkTreeIter           iter, child_iter;
+	NautilusIconInfo *icon_info;
+	int icon_size;
 
-	pixbuf = nautilus_icon_factory_get_pixbuf_from_name_with_stock_size (icon, NULL, GTK_ICON_SIZE_MENU, NULL);
+	icon_size = nautilus_get_icon_size_for_stock_size (GTK_ICON_SIZE_MENU);
+	icon_info = nautilus_icon_info_lookup (icon, icon_size);
+
+	pixbuf = nautilus_icon_info_get_pixbuf_at_size (icon_info, icon_size);
+	g_object_unref (icon_info);
 	gtk_list_store_append (sidebar->store, &iter);
 	gtk_list_store_set (sidebar->store, &iter,
 			    PLACES_SIDEBAR_COLUMN_ICON, pixbuf,
@@ -205,6 +214,7 @@ add_place (NautilusPlacesSidebar *sidebar,
 			    PLACES_SIDEBAR_COLUMN_URI, uri,
 			    PLACES_SIDEBAR_COLUMN_DRIVE, drive,
 			    PLACES_SIDEBAR_COLUMN_VOLUME, volume,
+			    PLACES_SIDEBAR_COLUMN_MOUNT, mount,
 			    PLACES_SIDEBAR_COLUMN_ROW_TYPE, place_type,
 			    PLACES_SIDEBAR_COLUMN_INDEX, index,
 			    -1);
@@ -221,16 +231,21 @@ add_place (NautilusPlacesSidebar *sidebar,
 static void
 update_places (NautilusPlacesSidebar *sidebar)
 {
-	NautilusBookmark      *bookmark;
-	GtkTreeSelection      *selection;
-	GtkTreeIter           iter, last_iter;
-	GnomeVFSVolumeMonitor *volume_monitor;
-	GList 		      *volumes, *l, *ll;
-	GnomeVFSVolume	      *volume;
-	GList		      *drives;
-	GnomeVFSDrive	      *drive;
-	int 		      bookmark_count, index;
-	char 		      *location, *icon, *mount_uri, *name, *desktop_path;
+	NautilusBookmark *bookmark;
+	GtkTreeSelection *selection;
+	GtkTreeIter iter, last_iter;
+	GVolumeMonitor *volume_monitor;
+	GList *mounts, *l, *ll;
+	GMount *mount;
+	GList *drives;
+	GDrive *drive;
+	GList *volumes;
+	GVolume *volume;
+	int bookmark_count, index;
+	char *location, *mount_uri, *name, *desktop_path;
+	GIcon *icon;
+	GFile *root;
+	
 		
 	selection = gtk_tree_view_get_selection (sidebar->tree_view);
 	gtk_list_store_clear (sidebar->store);
@@ -244,9 +259,11 @@ update_places (NautilusPlacesSidebar *sidebar)
 
 		mount_uri = nautilus_get_home_directory_uri ();
 		display_name = g_filename_display_basename (g_get_home_dir ());
+		icon = g_themed_icon_new ("gnome-fs-home");
 		last_iter = add_place (sidebar, PLACES_BUILT_IN,
-				       display_name, "gnome-fs-home",
-				       mount_uri, NULL, NULL, 0);
+				       display_name, icon,
+				       mount_uri, NULL, NULL, NULL, 0);
+		g_object_unref (icon);
 		g_free (display_name);
 		if (strcmp (location, mount_uri) == 0) {
 			gtk_tree_selection_select_iter (selection, &last_iter);
@@ -254,10 +271,12 @@ update_places (NautilusPlacesSidebar *sidebar)
 		g_free (mount_uri);
 	}
 
-	mount_uri = gnome_vfs_get_uri_from_local_path (desktop_path);
+	mount_uri = g_filename_to_uri (desktop_path, NULL, NULL);
+	icon = g_themed_icon_new ("gnome-fs-desktop");
 	last_iter = add_place (sidebar, PLACES_BUILT_IN,
-			       _("Desktop"), "gnome-fs-desktop",
-			       mount_uri, NULL, NULL, 0);
+			       _("Desktop"), icon,
+			       mount_uri, NULL, NULL, NULL, 0);
+	g_object_unref (icon);
 	if (strcmp (location, mount_uri) == 0) {
 		gtk_tree_selection_select_iter (selection, &last_iter);
 	}	
@@ -265,101 +284,164 @@ update_places (NautilusPlacesSidebar *sidebar)
 	g_free (desktop_path);
 	
  	mount_uri = "file:///"; /* No need to strdup */
+	icon = g_themed_icon_new ("gnome-dev-harddisk");
 	last_iter = add_place (sidebar, PLACES_BUILT_IN,
-			       _("File System"), "gnome-dev-harddisk",
-			       mount_uri, NULL, NULL, 0);
+			       _("File System"), icon,
+			       mount_uri, NULL, NULL, NULL, 0);
+	g_object_unref (icon);
 	if (strcmp (location, mount_uri) == 0) {
 		gtk_tree_selection_select_iter (selection, &last_iter);
 	}
 
-	/* for all drives add all its volumes */
+	volume_monitor = sidebar->volume_monitor;
 
-	volume_monitor = gnome_vfs_get_volume_monitor ();
-	drives = gnome_vfs_volume_monitor_get_connected_drives (volume_monitor);
-	drives = g_list_sort (drives, (GCompareFunc)gnome_vfs_drive_compare);
+	/* first go through all connected drives */
+	drives = g_volume_monitor_get_connected_drives (volume_monitor);
 	for (l = drives; l != NULL; l = l->next) {
 		drive = l->data;
-		if (!gnome_vfs_drive_is_user_visible (drive)) {
-			gnome_vfs_drive_unref (drive);
-			continue;
-		}
-		if (gnome_vfs_drive_is_mounted (drive)) {
-			/* The drive is mounted, add all its volumes */
-			volumes = gnome_vfs_drive_get_mounted_volumes (drive);
-			volumes = g_list_sort (volumes, (GCompareFunc)gnome_vfs_volume_compare);
+
+		volumes = g_drive_get_volumes (drive);
+		if (volumes != NULL) {
 			for (ll = volumes; ll != NULL; ll = ll->next) {
 				volume = ll->data;
-				if (!gnome_vfs_volume_is_user_visible (volume)) {
-					gnome_vfs_volume_unref (volume);
-					continue;
+				mount = g_volume_get_mount (volume);
+				if (mount != NULL) {
+					/* Show mounted volume in the sidebar */
+					icon = g_mount_get_icon (mount);
+					root = g_mount_get_root (mount);
+					mount_uri = g_file_get_uri (root);
+					g_object_unref (root);
+					name = g_mount_get_name (mount);
+					last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
+							       name, icon, mount_uri,
+							       drive, volume, mount, 0);
+					if (strcmp (location, mount_uri) == 0) {
+						gtk_tree_selection_select_iter (selection, &last_iter);
+					}
+					g_object_unref (mount);
+					g_object_unref (icon);
+					g_free (name);
+					g_free (mount_uri);
+				} else {
+					/* Do show the unmounted volumes in the sidebar;
+					 * this is so the user can mount it (in case automounting
+					 * is off).
+					 *
+					 * Also, even if automounting is enabled, this gives a visual
+					 * cue that the user should remember to yank out the media if
+					 * he just unmounted it.
+					 */
+					icon = g_volume_get_icon (volume);
+					name = g_volume_get_name (volume);
+					last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
+							       name, icon, NULL,
+							       drive, volume, NULL, 0);
+					g_object_unref (icon);
+					g_free (name);
 				}
-				icon = gnome_vfs_volume_get_icon (volume);
-				mount_uri = gnome_vfs_volume_get_activation_uri (volume);
-				name = gnome_vfs_volume_get_display_name (volume);
-				last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
-						       name, icon, mount_uri,
-						       drive, volume, 0);
-				if (strcmp (location, mount_uri) == 0) {
-					gtk_tree_selection_select_iter (selection, &last_iter);
-				}
-				gnome_vfs_volume_unref (volume);
-				g_free (icon);
-				g_free (name);
-				g_free (mount_uri);
+				g_object_unref (volume);
 			}
-			g_list_free (volumes);
 		} else {
-			/* The drive is unmounted but visible, add it.
-			 * This is for drives like floppy that can't be
-			 * auto-mounted */
-			icon = gnome_vfs_drive_get_icon (drive);
-			name = gnome_vfs_drive_get_display_name (drive);
-			last_iter = add_place (sidebar, PLACES_BUILT_IN,
-					       name, icon, NULL,
-					       drive, NULL, 0);
-			g_free (icon);
-			g_free (name);
-		}		
-		gnome_vfs_drive_unref (drive);
+			if (g_drive_is_media_removable (drive) && !g_drive_is_media_check_automatic (drive)) {
+				/* If the drive has no mountable volumes and we cannot detect media change.. we
+				 * display the drive in the sidebar so the user can manually poll the drive by
+				 * right clicking and selecting "Rescan..."
+				 *
+				 * This is mainly for drives like floppies where media detection doesn't
+				 * work.. but it's also for human beings who like to turn off media detection
+				 * in the OS to save battery juice.
+				 */
+				icon = g_drive_get_icon (drive);
+				name = g_drive_get_name (drive);
+				last_iter = add_place (sidebar, PLACES_BUILT_IN,
+						       name, icon, NULL,
+						       drive, NULL, NULL, 0);
+				g_object_unref (icon);
+				g_free (name);
+			}
+		}
+		g_object_unref (drive);
 	}
 	g_list_free (drives);
 
-	/* add mounted volumes that has no drive (ftp, sftp,...) */
-
-	volumes = gnome_vfs_volume_monitor_get_mounted_volumes (volume_monitor);
-	volumes = g_list_sort (volumes, (GCompareFunc)gnome_vfs_volume_compare);
+	/* add all volumes that is not associated with a drive */
+	volumes = g_volume_monitor_get_volumes (volume_monitor);
 	for (l = volumes; l != NULL; l = l->next) {
 		volume = l->data;
-		drive = gnome_vfs_volume_get_drive (volume);
-		if (!gnome_vfs_volume_is_user_visible (volume) || drive != NULL) {
-		    	gnome_vfs_drive_unref (drive);
-			gnome_vfs_volume_unref (volume);
+		drive = g_volume_get_drive (volume);
+		if (drive != NULL) {
+		    	g_object_unref (volume);
+			g_object_unref (drive);
 			continue;
 		}
-		icon = gnome_vfs_volume_get_icon (volume);
-		mount_uri = gnome_vfs_volume_get_activation_uri (volume);
-		name = gnome_vfs_volume_get_display_name (volume);
-		last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
-				       name, icon, mount_uri,
-				       NULL, volume, 0);
-		if (strcmp (location, mount_uri) == 0) {
-			gtk_tree_selection_select_iter (selection, &last_iter);
+		mount = g_volume_get_mount (volume);
+		if (mount != NULL) {
+			icon = g_mount_get_icon (mount);
+			root = g_mount_get_root (mount);
+			mount_uri = g_file_get_uri (root);
+			g_object_unref (root);
+			name = g_mount_get_name (mount);
+			last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
+					       name, icon, mount_uri,
+					       NULL, volume, mount, 0);
+			if (strcmp (location, mount_uri) == 0) {
+				gtk_tree_selection_select_iter (selection, &last_iter);
+			}
+			g_object_unref (mount);
+			g_object_unref (icon);
+			g_free (name);
+			g_free (mount_uri);
+		} else {
+			/* see comment above in why we add an icon for an unmounted mountable volume */
+			icon = g_volume_get_icon (volume);
+			name = g_volume_get_name (volume);
+			last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
+					       name, icon, NULL,
+					       NULL, volume, NULL, 0);
+			g_object_unref (icon);
+			g_free (name);
 		}
-		gnome_vfs_volume_unref (volume);
-		g_free (icon);
-		g_free (name);
-		g_free (mount_uri);
+		g_object_unref (volume);
 	}
 	g_list_free (volumes);
 
+	/* add mounts that has no volume (/etc/mtab mounts, ftp, sftp,...) */
+	mounts = g_volume_monitor_get_mounts (volume_monitor);
+	for (l = mounts; l != NULL; l = l->next) {
+		mount = l->data;
+		volume = g_mount_get_volume (mount);
+		if (volume != NULL) {
+		    	g_object_unref (volume);
+			g_object_unref (mount);
+			continue;
+		}
+		icon = g_mount_get_icon (mount);
+		root = g_mount_get_root (mount);
+		mount_uri = g_file_get_uri (root);
+		g_object_unref (root);
+		name = g_mount_get_name (mount);
+		last_iter = add_place (sidebar, PLACES_MOUNTED_VOLUME,
+				       name, icon, mount_uri,
+				       NULL, NULL, mount, 0);
+		if (strcmp (location, mount_uri) == 0) {
+			gtk_tree_selection_select_iter (selection, &last_iter);
+		}
+		g_object_unref (mount);
+		g_object_unref (icon);
+		g_free (name);
+		g_free (mount_uri);
+	}
+	g_list_free (mounts);
+
 	mount_uri = "trash:///"; /* No need to strdup */
-	icon = nautilus_trash_monitor_is_empty () ? "user-trash" : "user-trash-full";
+	icon = nautilus_trash_monitor_get_icon ();
 	last_iter = add_place (sidebar, PLACES_BUILT_IN,
 			       _("Trash"), icon, mount_uri,
-			       NULL, NULL, 0);
+			       NULL, NULL, NULL, 0);
 	if (strcmp (location, mount_uri) == 0) {
 		gtk_tree_selection_select_iter (selection, &last_iter);
 	}
+	g_object_unref (icon);
 
 	/* add separator */
 
@@ -383,24 +465,17 @@ update_places (NautilusPlacesSidebar *sidebar)
 		mount_uri = nautilus_bookmark_get_uri (bookmark);
 		last_iter = add_place (sidebar, PLACES_BOOKMARK,
 				       name, icon, mount_uri,
-				       NULL, NULL, index);
+				       NULL, NULL, NULL, index);
 		if (strcmp (location, mount_uri) == 0) {
 			gtk_tree_selection_select_iter (selection, &last_iter);
 		}
 		g_free (name);
-		g_free (icon);
+		g_object_unref (icon);
 		g_free (mount_uri);
 	}
 	g_free (location);
 }
 
-static gboolean
-update_places_cb (gpointer data)
-{
-	update_places (NAUTILUS_PLACES_SIDEBAR (data));
-	return FALSE;
-}
-
 static gboolean
 nautilus_shortcuts_row_separator_func (GtkTreeModel *model,
 				       GtkTreeIter  *iter,
@@ -417,58 +492,76 @@ nautilus_shortcuts_row_separator_func (GtkTreeModel *model,
   	return FALSE;
 }
 
+static void
+mount_added_callback (GVolumeMonitor *volume_monitor,
+		      GMount *mount,
+		      NautilusPlacesSidebar *sidebar)
+{
+	update_places (sidebar);
+}
 
 static void
-volume_mounted_callback (GnomeVFSVolumeMonitor *volume_monitor,
-			 GnomeVFSVolume *volume,
+mount_removed_callback (GVolumeMonitor *volume_monitor,
+			GMount *mount,
+			NautilusPlacesSidebar *sidebar)
+{
+	update_places (sidebar);
+}
+
+static void
+mount_changed_callback (GVolumeMonitor *volume_monitor,
+			GMount *mount,
+			NautilusPlacesSidebar *sidebar)
+{
+	update_places (sidebar);
+}
+
+static void
+volume_added_callback (GVolumeMonitor *volume_monitor,
+		       GVolume *volume,
+		       NautilusPlacesSidebar *sidebar)
+{
+	update_places (sidebar);
+}
+
+static void
+volume_removed_callback (GVolumeMonitor *volume_monitor,
+			 GVolume *volume,
 			 NautilusPlacesSidebar *sidebar)
 {
 	update_places (sidebar);
 }
 
 static void
-volume_unmounted_callback (GnomeVFSVolumeMonitor *volume_monitor,
-			   GnomeVFSVolume *volume,
-			   NautilusPlacesSidebar *sidebar)
+volume_changed_callback (GVolumeMonitor *volume_monitor,
+			 GVolume *volume,
+			 NautilusPlacesSidebar *sidebar)
 {
-	/* At this point the volume still appears to be mounted.
-	 * GnomeVFS will update its list after the signal finished
-	 * to emit, so we delay the update.
-	 */
-	g_idle_add (update_places_cb, sidebar);
+	update_places (sidebar);
 }
 
 static void
-drive_disconnected_callback (GnomeVFSVolumeMonitor *volume_monitor,
-			     GnomeVFSDrive         *drive,
+drive_disconnected_callback (GVolumeMonitor *volume_monitor,
+			     GDrive         *drive,
 			     NautilusPlacesSidebar *sidebar)
 {
 	update_places (sidebar);
 }
 
 static void
-drive_connected_callback (GnomeVFSVolumeMonitor *volume_monitor,
-			  GnomeVFSDrive         *drive,
+drive_connected_callback (GVolumeMonitor *volume_monitor,
+			  GDrive         *drive,
 			  NautilusPlacesSidebar *sidebar)
 {
 	update_places (sidebar);
 }
 
 static void
-volume_op_callback (gboolean succeeded,
-		    char *error,
-		    char *detailed_error,
-		    gpointer user_data)
+drive_changed_callback (GVolumeMonitor *volume_monitor,
+			GDrive         *drive,
+			NautilusPlacesSidebar *sidebar)
 {
-	if (!succeeded) {
-		if (*error == 0 &&
-		    detailed_error != NULL && *detailed_error == 0) {
-			/* This means the command displays its own errors */
-			return;
-		}
-		eel_show_error_dialog_with_details (error, NULL,
-						    detailed_error, NULL);
-	}
+	update_places (sidebar);
 }
 
 static void
@@ -674,7 +767,7 @@ can_accept_items_as_bookmarks (const GList *items)
 	 */
 	for (max = 100; items != NULL && max >= 0; items = items->next, max--) {
 		uri = ((NautilusDragSelectionItem *)items->data)->uri;
-		file = nautilus_file_get (uri);
+		file = nautilus_file_get_by_uri (uri);
 		if (!can_accept_file_as_bookmark (file)) {
 			nautilus_file_unref (file);
 			return FALSE;
@@ -766,20 +859,24 @@ drag_leave_callback (GtkTreeView *tree_view,
 /* Parses a "text/uri-list" string and inserts its URIs as bookmarks */
 static void
 bookmarks_drop_uris (NautilusPlacesSidebar *sidebar,
-		     const char            *data,
+		     GtkSelectionData      *selection_data,
 		     int                    position)
 {
 	NautilusBookmark *bookmark;
 	NautilusFile *file;
-	char *uri, *name, *name_truncated;
+	char *uri, *name;
 	char **uris;
 	int i;
+	GFile *location;
+	GIcon *icon;
 	
-	uris = g_uri_list_extract_uris (data);
+	uris = gtk_selection_data_get_uris (selection_data);
+	if (!uris)
+		return;
 	
 	for (i = 0; uris[i]; i++) {
 		uri = uris[i];
-		file = nautilus_file_get (uri);
+		file = nautilus_file_get_by_uri (uri);
 
 		if (!can_accept_file_as_bookmark (file)) {
 			nautilus_file_unref (file);
@@ -787,20 +884,22 @@ bookmarks_drop_uris (NautilusPlacesSidebar *sidebar,
 		}
 
 		uri = nautilus_file_get_drop_target_uri (file);
+		location = g_file_new_for_uri (uri);
 		nautilus_file_unref (file);
 
-		name = nautilus_compute_title_for_uri (uri);
-		name_truncated = eel_truncate_text_for_menu_item (name);	
+		name = nautilus_compute_title_for_location (location);
 
-		bookmark = nautilus_bookmark_new_with_icon (uri, name_truncated,
-							    FALSE, "gnome-fs-directory");
+		icon = g_themed_icon_new ("gnome-fs-directory");
+		bookmark = nautilus_bookmark_new_with_icon (location, name,
+							    FALSE, icon);
+		g_object_unref (icon);
 		
 		if (!nautilus_bookmark_list_contains (sidebar->bookmarks, bookmark)) {
 			nautilus_bookmark_list_insert_item (sidebar->bookmarks, bookmark, position++);
 		}
 
+		g_object_unref (location);
 		g_object_unref (bookmark);
-		g_free (name_truncated);
 		g_free (name);
 		g_free (uri);
 	}
@@ -964,7 +1063,7 @@ drag_data_received_callback (GtkWidget *widget,
 	
 		switch (info) {
 		case TEXT_URI_LIST:
-			bookmarks_drop_uris (sidebar, selection_data->data, position);
+			bookmarks_drop_uris (sidebar, selection_data, position);
 			success = TRUE;
 			break;
 		case GTK_TREE_MODEL_ROW:
@@ -1052,53 +1151,53 @@ bookmarks_popup_menu_detach_cb (GtkWidget *attach_widget,
 	sidebar->popup_menu_mount_item = NULL;
 	sidebar->popup_menu_unmount_item = NULL;
 	sidebar->popup_menu_eject_item = NULL;
+	sidebar->popup_menu_rescan_item = NULL;
 	sidebar->popup_menu_format_item = NULL;
 	sidebar->popup_menu_empty_trash_item = NULL;
 }
 
-static gboolean
-eject_for_type (GnomeVFSDeviceType type)
-{
-	switch (type) {
-	case GNOME_VFS_DEVICE_TYPE_CDROM:
-	case GNOME_VFS_DEVICE_TYPE_ZIP:
-	case GNOME_VFS_DEVICE_TYPE_JAZ:
-		return TRUE;
-	default:
-		return FALSE;
-	}
-}
-
 static void
-check_visibility (GnomeVFSVolume *volume,
-		  GnomeVFSDrive  *drive,
-		  gboolean       *show_mount,
-		  gboolean       *show_unmount,
-		  gboolean       *show_eject,
-		  gboolean       *show_format)
+check_visibility (GMount           *mount,
+		  GVolume          *volume,
+		  GDrive           *drive,
+		  gboolean         *show_mount,
+		  gboolean         *show_unmount,
+		  gboolean         *show_eject,
+		  gboolean         *show_rescan,
+		  gboolean         *show_format)
 {
 	*show_mount = FALSE;
 	*show_unmount = FALSE;
 	*show_eject = FALSE;
 	*show_format = FALSE;
+	*show_rescan = FALSE;
+
+	if (drive != NULL) {
+		*show_eject = g_drive_can_eject (drive);
+
+		if (g_drive_is_media_removable (drive) &&
+		    !g_drive_is_media_check_automatic (drive) && 
+		    g_drive_can_poll_for_media (drive))
+			*show_rescan = TRUE;
+	}
 
 	if (volume != NULL) {
-		*show_unmount = TRUE;
-		*show_eject = eject_for_type (gnome_vfs_volume_get_device_type (volume));
-	} else if (drive != NULL) {
-		*show_eject = eject_for_type (gnome_vfs_drive_get_device_type (drive));
-		if (gnome_vfs_drive_is_mounted (drive)) {
-			*show_unmount = TRUE;
-		} else {
-			*show_mount = TRUE;
-		}
+		*show_eject |= g_volume_can_eject (volume);
+		if (mount == NULL)
+			*show_mount = g_volume_can_mount (volume);
+	}
+	
+	if (mount != NULL) {
+		*show_unmount = g_mount_can_unmount (mount);
+		*show_eject |= g_mount_can_eject (mount);
+	}
 
-		if (gnome_vfs_drive_get_device_type (drive) == GNOME_VFS_DEVICE_TYPE_FLOPPY &&
-		    !gnome_vfs_drive_is_mounted (drive) &&
+#ifdef TODO_GIO
+		if (something &&
 		    g_find_program_in_path ("gfloppy")) {
 			*show_format = TRUE;
 		}
-	}
+#endif		
 }
 
 static void
@@ -1106,11 +1205,13 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 {
 	GtkTreeIter iter;
 	PlaceType type; 
-	GnomeVFSDrive *drive = NULL;
-	GnomeVFSVolume *volume = NULL;
+	GDrive *drive = NULL;
+	GVolume *volume = NULL;
+	GMount *mount = NULL;
 	gboolean show_mount;
 	gboolean show_unmount;
 	gboolean show_eject;
+	gboolean show_rescan;
 	gboolean show_format;
 	gboolean show_empty_trash;
 	char *uri = NULL;
@@ -1125,7 +1226,8 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 		gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
 				    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &type,
 				    PLACES_SIDEBAR_COLUMN_DRIVE, &drive,
- 				    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
+				    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
+ 				    PLACES_SIDEBAR_COLUMN_MOUNT, &mount,
 				    PLACES_SIDEBAR_COLUMN_URI, &uri,
 				    -1);
 	}
@@ -1134,14 +1236,12 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 	gtk_widget_set_sensitive (sidebar->popup_menu_rename_item, (type == PLACES_BOOKMARK));
 	gtk_widget_set_sensitive (sidebar->popup_menu_empty_trash_item, !nautilus_trash_monitor_is_empty ());
 
- 	check_visibility (volume, drive,
- 			  &show_mount, &show_unmount, &show_eject, &show_format);	
+ 	check_visibility (mount, volume, drive,
+ 			  &show_mount, &show_unmount, &show_eject, &show_rescan, &show_format);
 
-	/* We don't want both eject and unmount, since eject
- 	   unmounts too */
- 	if (show_eject) {
- 		show_unmount = FALSE;
-  	}
+	/* We actually want both eject and unmount since eject will unmount all volumes. 
+	 * TODO: hide unmount if the drive only has a single mountable volume 
+	 */
 
 	show_empty_trash = (uri != NULL) &&
 			   (!strcmp (uri, "trash:///"));
@@ -1151,6 +1251,7 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 	eel_gtk_widget_set_shown (sidebar->popup_menu_mount_item, show_mount);
 	eel_gtk_widget_set_shown (sidebar->popup_menu_unmount_item, show_unmount);
 	eel_gtk_widget_set_shown (sidebar->popup_menu_eject_item, show_eject);
+	eel_gtk_widget_set_shown (sidebar->popup_menu_rescan_item, show_rescan);
 	eel_gtk_widget_set_shown (sidebar->popup_menu_format_item, show_format);
 	eel_gtk_widget_set_shown (sidebar->popup_menu_empty_trash_item, show_empty_trash);
 
@@ -1165,6 +1266,33 @@ bookmarks_selection_changed_cb (GtkTreeSelection      *selection,
 	bookmarks_check_popup_sensitivity (sidebar);
 }
 
+static void
+volume_mount_cb (GObject *source_object,
+		 GAsyncResult *res,
+		 gpointer user_data)
+{
+	GMountOperation *mount_op = user_data;
+	GError *error;
+	char *primary;
+	char *name;
+
+	error = NULL;
+	if (!g_volume_mount_finish (G_VOLUME (source_object), res, &error)) {
+		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
+			name = g_volume_get_name (G_VOLUME (source_object));
+			primary = g_strdup_printf (_("Unable to mount %s"), name);
+			g_free (name);
+			eel_show_error_dialog (primary,
+					       error->message,
+					       NULL);
+			g_free (primary);
+		}
+		g_error_free (error);
+	}
+
+	g_object_unref (mount_op);
+}
+
 static void
 open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 			GtkTreeModel	      *model,
@@ -1172,6 +1300,7 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 			gboolean	      open_in_new_window)
 {
 	GtkTreeIter iter;
+	GFile *location;
 	char *uri;
 
 	if (!path) {
@@ -1188,35 +1317,32 @@ open_selected_bookmark (NautilusPlacesSidebar *sidebar,
 		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
 				    "activate from places sidebar window=%p: %s",
 				    sidebar->window, uri);
+		location = g_file_new_for_uri (uri);
 		/* Navigate to the clicked location */
 		if (!open_in_new_window) {
-			nautilus_window_info_open_location (sidebar->window, uri,
+			nautilus_window_info_open_location (sidebar->window, location,
 							    NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
 							    0, NULL);
 		} else {
 			NautilusWindow *cur, *new;
+			
 			cur = NAUTILUS_WINDOW (sidebar->window);
 			new = nautilus_application_create_navigation_window (cur->application,
 									     NULL,
 									     gtk_window_get_screen (GTK_WINDOW (cur)));
-			nautilus_window_go_to (new, uri);
+			nautilus_window_go_to (new, location);
 		}
+		g_object_unref (location);
 		g_free (uri);
 
 	} else {
-		GnomeVFSDrive *drive;
-		gtk_tree_model_get (model, &iter, PLACES_SIDEBAR_COLUMN_DRIVE, &drive, -1);
-		if (drive != NULL) {
-			char *path;
-
-			path = gnome_vfs_drive_get_device_path (drive);
-			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-					    "activate drive from places sidebar window=%p: %s",
-					    sidebar->window, path);
-			g_free (path);
-
-			gnome_vfs_drive_mount (drive, volume_op_callback, sidebar);
-			gnome_vfs_drive_unref (drive);
+		GVolume *volume;
+		gtk_tree_model_get (model, &iter, PLACES_SIDEBAR_COLUMN_VOLUME, &volume, -1);
+		if (volume != NULL) {
+			GMountOperation *mount_op;
+			mount_op = g_mount_operation_new ();
+			g_volume_mount (volume, mount_op, NULL, volume_mount_cb, mount_op);
+			g_object_unref (volume);
 		}
 	}
 }
@@ -1319,7 +1445,190 @@ mount_shortcut_cb (GtkMenuItem           *item,
 		   NautilusPlacesSidebar *sidebar)
 {
 	GtkTreeIter iter;
-	GnomeVFSDrive *drive;
+	GVolume *volume;
+
+	if (!get_selected_iter (sidebar, &iter)) {
+		return;
+	}
+
+	gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
+			    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
+			    -1);
+
+	if (volume != NULL) {
+		GMountOperation *mount_op;
+		mount_op = g_mount_operation_new ();
+		g_volume_mount (volume, mount_op, NULL, volume_mount_cb, mount_op);
+		g_object_unref (volume);
+	}
+}
+
+static void
+unmount_shortcut_cb (GtkMenuItem           *item,
+		     NautilusPlacesSidebar *sidebar)
+{
+	GtkTreeIter iter;
+	GMount *mount;
+	GVolume *volume;
+
+	if (!get_selected_iter (sidebar, &iter)) {
+		return;
+	}
+
+	gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
+			    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
+			    PLACES_SIDEBAR_COLUMN_MOUNT, &mount,
+			    -1);
+
+	if (mount != NULL) {
+		GtkWidget *toplevel;
+		
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (sidebar->tree_view));
+		nautilus_file_operations_unmount_mount (GTK_WINDOW (toplevel),
+							mount, FALSE);
+	}
+	if (mount != NULL) {
+		g_object_unref (mount);
+	}
+	if (volume != NULL) {
+		g_object_unref (volume);
+	}
+}
+
+static void
+drive_eject_cb (GObject *source_object,
+		GAsyncResult *res,
+		gpointer user_data)
+{
+	GError *error;
+	char *primary;
+	char *name;
+	error = NULL;
+	if (!g_drive_eject_finish (G_DRIVE (source_object), res, &error)) {
+		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
+			name = g_drive_get_name (G_DRIVE (source_object));
+			primary = g_strdup_printf (_("Unable to eject %s"), name);
+			g_free (name);
+			eel_show_error_dialog (primary,
+					       error->message,
+				       NULL);
+			g_free (primary);
+		}
+		g_error_free (error);
+	}
+}
+
+static void
+volume_eject_cb (GObject *source_object,
+		GAsyncResult *res,
+		gpointer user_data)
+{
+	GError *error;
+	char *primary;
+	char *name;
+	error = NULL;
+	if (!g_volume_eject_finish (G_VOLUME (source_object), res, &error)) {
+		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
+			name = g_volume_get_name (G_VOLUME (source_object));
+			primary = g_strdup_printf (_("Unable to eject %s"), name);
+			g_free (name);
+			eel_show_error_dialog (primary,
+					       error->message,
+					       NULL);
+			g_free (primary);
+		}
+		g_error_free (error);
+	}
+}
+
+static void
+mount_eject_cb (GObject *source_object,
+		GAsyncResult *res,
+		gpointer user_data)
+{
+	GError *error;
+	char *primary;
+	char *name;
+	error = NULL;
+	if (!g_mount_eject_finish (G_MOUNT (source_object), res, &error)) {
+		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
+			name = g_mount_get_name (G_MOUNT (source_object));
+			primary = g_strdup_printf (_("Unable to eject %s"), name);
+			g_free (name);
+			eel_show_error_dialog (primary,
+					       error->message,
+					       NULL);
+			g_free (primary);
+		}
+		g_error_free (error);
+	}
+}
+
+static void
+eject_shortcut_cb (GtkMenuItem           *item,
+		   NautilusPlacesSidebar *sidebar)
+{
+	GtkTreeIter iter;
+	GMount *mount;
+	GVolume *volume;
+	GDrive *drive;
+
+	if (!get_selected_iter (sidebar, &iter)) {
+		return;
+	}
+
+	gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
+			    PLACES_SIDEBAR_COLUMN_MOUNT, &mount,
+			    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
+			    PLACES_SIDEBAR_COLUMN_DRIVE, &drive,
+			    -1);
+
+	if (mount != NULL) {
+		g_mount_eject (mount, 0, NULL, mount_eject_cb, NULL);
+	} else if (volume != NULL) {
+		g_volume_eject (volume, 0, NULL, volume_eject_cb, NULL);
+	} else if (drive != NULL) {
+		g_drive_eject (drive, 0, NULL, drive_eject_cb, NULL);
+	}
+
+	if (mount != NULL)
+		g_object_unref (mount);
+	if (volume != NULL)
+		g_object_unref (volume);
+	if (drive != NULL)
+		g_object_unref (drive);
+}
+
+static void
+drive_poll_for_media_cb (GObject *source_object,
+			 GAsyncResult *res,
+			 gpointer user_data)
+{
+	GError *error;
+	char *primary;
+	char *name;
+
+	error = NULL;
+	if (!g_drive_poll_for_media_finish (G_DRIVE (source_object), res, &error)) {
+		if (error->code != G_IO_ERROR_FAILED_HANDLED) {
+			name = g_drive_get_name (G_DRIVE (source_object));
+			primary = g_strdup_printf (_("Unable to poll %s for media changes"), name);
+			g_free (name);
+			eel_show_error_dialog (primary,
+					       error->message,
+					       NULL);
+			g_free (primary);
+		}
+		g_error_free (error);
+	}
+}
+
+static void
+rescan_shortcut_cb (GtkMenuItem           *item,
+		    NautilusPlacesSidebar *sidebar)
+{
+	GtkTreeIter iter;
+	GDrive  *drive;
 
 	if (!get_selected_iter (sidebar, &iter)) {
 		return;
@@ -1330,63 +1639,9 @@ mount_shortcut_cb (GtkMenuItem           *item,
 			    -1);
 
 	if (drive != NULL) {
-		gnome_vfs_drive_mount (drive, volume_op_callback, sidebar);
-		gnome_vfs_drive_unref (drive);
+		g_drive_poll_for_media (drive, NULL, drive_poll_for_media_cb, NULL);
 	}
-}
-
-static void
-unmount_shortcut_cb (GtkMenuItem           *item,
-		     NautilusPlacesSidebar *sidebar)
-{
-	GtkTreeIter iter;
-	GnomeVFSVolume *volume;
-	GnomeVFSDrive  *drive;
-
-	if (!get_selected_iter (sidebar, &iter)) {
-		return;
-	}
-
-	gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
-			    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
-			    PLACES_SIDEBAR_COLUMN_DRIVE, &drive,
-			    -1);
-
-	if (volume != NULL) {
-		nautilus_file_operations_unmount_volume (GTK_WIDGET (sidebar->tree_view),
-				volume, volume_op_callback, sidebar);
-	} else if (drive != NULL) {
-		nautilus_file_operations_unmount_drive (GTK_WIDGET (sidebar->tree_view),
-				drive, volume_op_callback, sidebar);
-	}
-	gnome_vfs_volume_unref (volume);
-	gnome_vfs_drive_unref (drive);
-}
-
-static void
-eject_shortcut_cb (GtkMenuItem           *item,
-		   NautilusPlacesSidebar *sidebar)
-{
-	GtkTreeIter iter;
-	GnomeVFSVolume *volume;
-	GnomeVFSDrive  *drive;
-
-	if (!get_selected_iter (sidebar, &iter)) {
-		return;
-	}
-
-	gtk_tree_model_get (GTK_TREE_MODEL (sidebar->store), &iter,
-			    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
-			    PLACES_SIDEBAR_COLUMN_DRIVE, &drive,
-			    -1);
-
-	if (volume != NULL) {
-		gnome_vfs_volume_eject (volume, volume_op_callback, sidebar);
-	} else if (drive != NULL) {
-		gnome_vfs_drive_eject (drive, volume_op_callback, sidebar);
-	}
-	gnome_vfs_volume_unref (volume);
-	gnome_vfs_drive_unref (drive);
+	g_object_unref (drive);
 }
 
 static void
@@ -1501,6 +1756,13 @@ bookmarks_build_popup_menu (NautilusPlacesSidebar *sidebar)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
 
+	item = gtk_menu_item_new_with_mnemonic (_("_Rescan"));
+	sidebar->popup_menu_rescan_item = item;
+	g_signal_connect (item, "activate",
+		    G_CALLBACK (rescan_shortcut_cb), sidebar);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
+
 	item = gtk_menu_item_new_with_mnemonic (_("_Format"));
 	sidebar->popup_menu_format_item = item;
 	g_signal_connect (item, "activate",
@@ -1618,6 +1880,8 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	GtkCellRenderer   *cell;
 	GtkTreeSelection  *selection;
 
+	sidebar->volume_monitor = g_volume_monitor_get ();
+	
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sidebar),
 					GTK_POLICY_NEVER,
 					GTK_POLICY_AUTOMATIC);
@@ -1658,8 +1922,9 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 	sidebar->store = gtk_list_store_new (PLACES_SIDEBAR_COLUMN_COUNT,
 					     G_TYPE_INT, 
 					     G_TYPE_STRING,
-					     GNOME_VFS_TYPE_DRIVE,
-					     GNOME_VFS_TYPE_VOLUME,
+					     G_TYPE_DRIVE,
+					     G_TYPE_VOLUME,
+					     G_TYPE_MOUNT,
 					     G_TYPE_STRING,
 					     GDK_TYPE_PIXBUF,
 					     G_TYPE_INT
@@ -1746,6 +2011,8 @@ nautilus_places_sidebar_finalize (GObject *object)
 		sidebar->store = NULL;
 	}
 
+	g_object_unref (sidebar->volume_monitor);
+	
 	eel_preferences_remove_callback (NAUTILUS_PREFERENCES_CLICK_POLICY,
 					 click_policy_changed_callback,
 					 sidebar);
@@ -1806,8 +2073,6 @@ static void
 nautilus_places_sidebar_set_parent_window (NautilusPlacesSidebar *sidebar,
 					    NautilusWindowInfo *window)
 {	
-	GnomeVFSVolumeMonitor *volume_monitor;
-
 	sidebar->window = window;
 	
 	sidebar->bookmarks = nautilus_window_info_get_bookmark_list (window);
@@ -1821,16 +2086,24 @@ nautilus_places_sidebar_set_parent_window (NautilusPlacesSidebar *sidebar,
 				 G_CALLBACK (loading_uri_callback),
 				 sidebar, 0);
 			 
-	volume_monitor = gnome_vfs_get_volume_monitor ();
-	
-	g_signal_connect_object (volume_monitor, "volume_mounted",
-				 G_CALLBACK (volume_mounted_callback), sidebar, 0);
-	g_signal_connect_object (volume_monitor, "volume_unmounted",
-				 G_CALLBACK (volume_unmounted_callback), sidebar, 0);
-	g_signal_connect_object (volume_monitor, "drive_disconnected",
+	g_signal_connect_object (sidebar->volume_monitor, "volume_added",
+				 G_CALLBACK (volume_added_callback), sidebar, 0);
+	g_signal_connect_object (sidebar->volume_monitor, "volume_removed",
+				 G_CALLBACK (volume_removed_callback), sidebar, 0);
+	g_signal_connect_object (sidebar->volume_monitor, "volume_changed",
+				 G_CALLBACK (volume_changed_callback), sidebar, 0);
+	g_signal_connect_object (sidebar->volume_monitor, "mount_added",
+				 G_CALLBACK (mount_added_callback), sidebar, 0);
+	g_signal_connect_object (sidebar->volume_monitor, "mount_removed",
+				 G_CALLBACK (mount_removed_callback), sidebar, 0);
+	g_signal_connect_object (sidebar->volume_monitor, "mount_changed",
+				 G_CALLBACK (mount_changed_callback), sidebar, 0);
+	g_signal_connect_object (sidebar->volume_monitor, "drive_disconnected",
 				 G_CALLBACK (drive_disconnected_callback), sidebar, 0);
-	g_signal_connect_object (volume_monitor, "drive_connected",
+	g_signal_connect_object (sidebar->volume_monitor, "drive_connected",
 				 G_CALLBACK (drive_connected_callback), sidebar, 0);
+	g_signal_connect_object (sidebar->volume_monitor, "drive_changed",
+				 G_CALLBACK (drive_changed_callback), sidebar, 0);
 
 	update_places (sidebar);
 }
